BLOCK = "block"
DIAMOND = "diamond"

CENTER = "center"
START = "start"

DEFAULT_TEXT_SIZE = 10
DEFAULT_FONT_FAMILY = "Helvetica"
DEFAULT_FONT_COLOR = "#000000"
DEFAULT_STROKE_WIDTH = "1"

ARRAY = "Array"
CELL = "mxCell"
GEO = "mxGeometry"
GRAPH = "mxGraphModel"
POINT = "mxPoint"
ROOT = "root"

NONE = "none"
CURVED = "curved=1;"
SHARP = "rounded=0;"

VERTICAL_ALIGN = {
    "t": "top",
    "b": "bottom",
    "c": "middle",
}
