BOX = "box"
BOX_3D = "box3d"
CDS = "cds"
CIRCLE = "circle"
COMPONENT = "component"
CYLINDER = "cylinder"
DIAMOND = "diamond"
DOUBLE_CIRCLE = "doublecircle"
DOUBLE_OCTAGON = "doubleoctagon"
EGG = "egg"
ELLIPSE = "ellipse"
FOLDER = "folder"
HEXAGON = "hexagon"
HOUSE = "house"
IMAGE = "image"
INV_HOUSE = "invhouse"
INV_TRAPEZIUM = "invtrapezium"
INV_TRIANGLE = "invtriangle"
LARROW = "larrow"
LPROMOTER = "lpromoter"
NOTE = "note"
OCTAGON = "octagon"
OVAL = "oval"
PARALLELOGRAM = "parallelogram"
PENTAGON = "pentagon"
PLAIN = "plain"
POLYGON = "polygon"
PROMOTER = "promoter"
RARROW = "rarrow"
RECT = "rect"
RECTANGLE = "rectangle"
RPROMOTER = "rpromoter"
SQUARE = "square"
STAR = "star"
TAB = "tab"
TRAPEZIUM = "trapezium"
TRIANGLE = "triangle"
UNDERLINE = "underline"
